@@ -15,18 +15,19 @@
 # You should have received a copy of the GNU General Public License
 # along with "sc8pr".  If not, see <http://www.gnu.org/licenses/>.
 
-version = 2, 1, 4
+
+version = 2, 2, 0
 print("sc8pr {}.{}.{}: https://dmaccarthy.github.io/sc8pr".format(*version))
 
-import sys, os, struct, zlib
-from math import hypot
+import sys, struct, zlib, io
+from math import hypot, sqrt
 import pygame
 import pygame.display as _pd
 from pygame.transform import flip as _pyflip
 from sc8pr._event import EventManager
-from sc8pr.geom import transform2d, positiveAngle, delta
-from sc8pr.util import CachedSurface, style, logError, sc8prData,\
-    tile, rgba, drawBorder, hasAlpha
+from sc8pr._cs import CoordSys
+from sc8pr.geom import transform2d, positiveAngle, delta, sigma, vmult
+from sc8pr.util import CachedSurface, style, logError, sc8prData, tile, rgba, drawBorder
 
 # Anchor point constants
 TOPLEFT = 0
@@ -50,8 +51,8 @@ CIRCLE = 0
 RECT = 1
 
 # pygame 1.9 <--> 2.0 compatibility
-SIZECHANGED = getattr(pygame, "WINDOWSIZECHANGED", -1)
-WINEXPOSED = getattr(pygame, "WINDOWEXPOSED", -1)
+SIZECHANGED = getattr(pygame, "WINDOWSIZECHANGED", None)
+WINEXPOSED = getattr(pygame, "WINDOWEXPOSED", None)
 
 
 class PixelData:
@@ -66,6 +67,7 @@ class PixelData:
         rgb = "RGBA", "RGB"
         self.compressed = False
         if type(img) is bytes: img = img[:-12], img[-12:]
+        elif type(img) is PixelData: img = img.raw()
         elif isinstance(img, Graphic):
             try: img = img.image
             except: img = img.snapshot()
@@ -90,6 +92,7 @@ class PixelData:
         else: raise NotImplementedError("Only RGB and RGBA modes are supported")
         self.codec = codec
         if compress: self.compress()
+        elif self.compressed: self.decompress()
 
     def compress(self):
         if not self.compressed:
@@ -140,9 +143,12 @@ class PixelData:
     @property
     def img(self): return Image(self.srf)
 
-    def pil(self, pil):
-        fn = lambda d,s,m: pil.Image.frombytes(m, s, d)
-        return self._image(fn)
+    @staticmethod
+    def _frombytes(d, s, m):
+        return sys.modules["PIL.Image"].frombytes(m, s, d)
+
+    @property
+    def pil(self): return self._image(PixelData._frombytes)
 
 
 class Graphic:
@@ -152,10 +158,13 @@ class Graphic:
     provide a 'image' property which gives a surface that Graphic.draw can use."""
     autoPositionOnResize = True
     _avgColor = None
+    _preserve = "xy", "size"
+    scrollable = True
     canvas = None
     pos = 0, 0
     anchor = CENTER
     angle = 0
+    hoverable = True
     focusable = False
     ondraw = None
     effects = None
@@ -166,6 +175,12 @@ class Graphic:
         "The key used when adding the instance to a canvas"
         return getattr(self, "_name", None)
 
+    @property
+    def _str_name(self):
+        name = self.name
+        if name is None: name = id(self)
+        return "{} '{}'".format(type(self).__name__, name)
+
     def anon(self):
         "Remove the instance's key (_name)"
         if hasattr(self, "_name"): del self._name
@@ -175,6 +190,15 @@ class Graphic:
         name = self.name
         return "<{} '{}'>".format(type(self).__name__, name if name else id(self))
 
+    def _config(self, error_mode=1, **kwargs):
+        "Set multiple instance properties; handle excpetions"
+        for i in kwargs.items():
+            try: setattr(self, *i)
+            except Exception as e:
+                if error_mode == 1: logError()
+                elif error_mode == 2: raise e
+        return self
+
     def config(self, **kwargs):
         "Set multiple instance properties"
         for i in kwargs.items(): setattr(self, *i)
@@ -193,7 +217,6 @@ class Graphic:
     @property
     def avgColor(self): return self._avgColor
 
-
 # Metrics
 
     @property
@@ -226,18 +249,48 @@ class Graphic:
     def height(self, height):
         self.resize((height * self.aspectRatio, height))
 
+    def scale(self, sx, sy=None):
+        w, h = self.size
+        self.size = sx * w, (sx if sy is None else sy) * h
+        return self
+
     @property
     def center(self):
         size = self.size
         return (size[0] - 1) / 2, (size[1] - 1) / 2
 
+    @property
+    def clockwise(self):
+        try: return self.canvas.clockwise
+        except: return True
+
+    @property
+    def xy(self):
+        "Position relative to canvas coordinate system"
+        try: return self.canvas.cs(*self.pos)
+        except: return self.pos
+
+    @xy.setter
+    def xy(self, pos):
+        try: self.pos = self.canvas.px(*pos)
+        except: self.pos = pos
+
+    @property
+    def theta(self):
+        "Graphic angle in the same sense as the coordinate system"
+        return self.angle if self.clockwise else -self.angle
+
+    @theta.setter
+    def theta(self, theta):
+        self.angle = theta if self.clockwise else -theta
+
     def blitPosition(self, offset, blitSize):
         "Return the position (top left) to which the graphic is drawn"
         x, y = self.pos
         x += offset[0]
         y += offset[1]
         a = self.anchor
-        if a:
+        if a: # (blitSize[*] - 1) makes robot jiggly
             x -= blitSize[0] * (a & 3) // 2
             y -= blitSize[1] * (a & 12) // 8
         return x, y
@@ -245,7 +298,7 @@ class Graphic:
     def calcBlitRect(self, blitSize):
         cv = self.canvas
         offset = cv.rect.topleft if cv else (0,0)
-        return pygame.Rect(self.blitPosition(offset, blitSize) + blitSize) 
+        return pygame.Rect(self.blitPosition(offset, blitSize), blitSize) ## !!!
 
     def relXY(self, pos):
         "Calculate coordinates relative to the graphic object"
@@ -266,10 +319,10 @@ class Graphic:
         else: r = self.rect
         return bool(r.collidepoint(pos))
 
-    def scaleVectors(self, fx, fy, attr=("pos", "vel", "acc")):
+    def scaleVectors(self, fx, fy, attr=("pos", "vel", "acc", "_scrollSize")):
         "Scale one or more 2-vectors"
         for a in attr:
-            try: # Skip undefined attributes
+            try: # Skip undefined or fixed attributes
                 x, y = getattr(self, a)
                 setattr(self, a, (x * fx, y * fy))
             except: pass
@@ -277,32 +330,47 @@ class Graphic:
 
 # Canvas interaction
 
-    def setCanvas(self, cv, key=None):
-        "Add the object to a canvas"
-#        key = self._name
+    def _setCanvas(self, cv, key):
         self.remove()
-        if key: 
+        if key:
             if key in cv and cv[key] is not self:
                 raise KeyError("Key '{}' is already in use".format(key))
             self._name = key
         self.canvas = cv
         cv._items.append(self)
+
+    def setCanvas(self, cv, key=None):
+        "Add the object to a canvas"
+        xy = self.xy
+        t = self.theta
+        self._setCanvas(cv, key)
+        self._config(0, xy=xy, theta=t)
         return self
 
-    def remove(self, deleteRect=True):
+    def remove(self, deleteRect=False):
         "Remove the instance from its canvas"
-#        if cv and self in cv._items:
+        cv = self.canvas
         try:
-            cv = self.canvas
-            self.anon()
             if deleteRect and hasattr(self, "rect"):
                 del self.rect
-            cv._items.remove(self)
+            if cv:
+                self.anon()
+                cv._items.remove(self)
+                if self.focussed: self.blur()
         except: pass
         return self
 
+    def toggle(self, name=None):
+        cv = self.canvas
+        if cv is not None:
+            if self in cv: self.remove()
+            else: self.setCanvas(cv, name)
+        return self
+
     @property
-    def layer(self): return self.canvas._items.index(self)
+    def layer(self):
+        try: return self.canvas._items.index(self)
+        except: pass
 
     @layer.setter
     def layer(self, n):
@@ -323,10 +391,27 @@ class Graphic:
     def focussed(self):
         return self is self.sketch.evMgr.focus
 
-    def blur(self):
+    def blur(self, trigger=False):
+        "Relinquish event focus to the sketch"
         sk = self.sketch
-        ev = sk.evMgr
-        if ev.focus is self: ev.focus = sk
+        if self is sk.evMgr.focus: sk.focus(trigger)
+        return self
+
+    def focus(self, trigger=False):
+        "Acquire event focus"
+        if not self.focusable:
+            raise AttributeError("Graphic instance is not focusable")
+        sk = self.sketch
+        if not sk:
+            raise KeyError("Cannot focus graphic that has not been added to the sketch")
+        evMgr = sk.evMgr
+        gr = evMgr.focus
+        trigger = trigger and gr is not self
+        if trigger:
+            ev = pygame.event.Event(pygame.USEREVENT, focus=self, hover=evMgr.hover)
+            evMgr.handle(gr, "onblur", ev)
+        evMgr.focus = self
+        if trigger: evMgr.handle(self, "onfocus", ev)
         return self
 
     @property
@@ -408,6 +493,11 @@ class Renderable(Graphic):
     def refresh(self):
         if self.stale: self.image
 
+    def remove(self, deleteRect=False): # !!!
+        super().remove(deleteRect)
+        self.stale = True
+        return self
+
     @property
     def avgColor(self):
         self.refresh()
@@ -438,7 +528,7 @@ class Renderable(Graphic):
         return self._srf.get_size()
 
     def resize(self, size):
-        self._size = size
+        self._size = size # ???
         self.stale = True
 
     @size.setter
@@ -615,6 +705,32 @@ class Image(Graphic):
             tiles += [Image(_pyflip(s.image, False, True)) for s in tiles]
         return tiles
 
+    def _cut(self, x=(), y=(), padding=0):
+        "Generate images by cutting the original"
+        srf = self.image
+        x0 = 0
+        for c in range(len(x) + 1):
+            y0 = 0
+            for r in range(len(y) + 1):
+                try: h = y[r] - y0
+                except: h = self.height - y0
+                try: w = x[c] - x0
+                except: w = self.width - x0
+                yield Image(srf.subsurface(x0+padding, y0+padding, w-2*padding, h-2*padding))
+                y0 += h
+            x0 += w
+
+    def cut(self, x=(), y=(), padding=0):
+        return list(self._cut(x, y, padding))
+
+    def flip(self, mode=HORIZONTAL):
+        "Create a new image by flipping an existing instance"
+        return Image(_pyflip(self.image, mode & HORIZONTAL, mode & VERTICAL))
+
+    def crop(self, *args):
+        "Create a new Image instance by cropping an existing image"
+        return Image(self.image.subsurface(pygame.Rect(*args)))
+
     @property
     def image(self):
         "Return a scaled and rotated surface"
@@ -625,24 +741,33 @@ class Image(Graphic):
         try: return bool(self.at(pos, self.rect).a)
         except: return False
 
-    def save(self, fn):
+    def save(self, fn):           
         pygame.image.save(self._srf.original, fn)
         return self
 
+    def _export(self, fn="a.png"):
+        b = io.BytesIO(b"")
+        pygame.image.save(self.image, b, fn)
+        b.seek(0)
+        return b
+
+    @property
+    def png(self): return self._export()
+
+    @property
+    def jpg(self): return self._export("a.jpg")
+        
     def copy(self): return Image(self.image.copy())
 
 
 class Canvas(Graphic):
     _border = rgba("black")
+    _clipArea = None
     weight = 0
-
-#    def __init__(self, image, bg=None):
-#        if type(image) is str: bg = Image(image, bg)
-#        self._size = bg.size if isinstance(bg, Image) else image
-#        self.bg = bg
-#        self._items = []
+    resizeContent = True
 
     def __init__(self, image, bg=None):
+        self.resetCS()
         mode = 0 if type(image) is str else 1 if isinstance(image, Image) else 2
         if mode == 2: # tuple or list
             size = image
@@ -656,6 +781,69 @@ class Canvas(Graphic):
         self._items = []
         self.bg = bg
 
+    @property
+    def clipArea(self): return self._clipArea
+
+    @clipArea.setter
+    def clipArea(self, r): self._clipArea = pygame.Rect(r)
+
+    @property
+    def _scroll(self): return 0, 0
+
+    @property
+    def clockwise(self):
+        ux, uy = self.units
+        return ux * uy > 0
+
+    @property
+    def units(self): return self._units
+
+    def px_delta(self, dx=0, dy=0):
+        ux, uy = self._units
+        return dx*ux, dy*uy
+
+    def cs_delta(self, dx=0, dy=0):
+        ux, uy = self._units
+        return dx/ux, dy/uy
+
+    @property
+    def unit(self):
+        ux, uy = self.units
+        return sqrt(abs(ux * uy))
+
+    def px(self, *pt): return delta(self._px(pt), self._scroll)
+    def cs(self, *pt): return self._cs(sigma(pt, self._scroll))
+    def px_list(self, *args): return [self.px(*pt) for pt in args]
+    def cs_list(self, *args): return [self.cs(*pt) for pt in args]
+
+    def attachCS(self, lrbt, margin=0, size=None):
+        "Attach a coordinate system to the canvas"
+        if size is None: size = getattr(self, "scrollSize", self.size)
+        self.coordSys = cs = CoordSys(lrbt, size, margin)
+        self._cs, self._px = cs._tr
+        self._units = delta(self.px(1, 1), self.px(0, 0))
+        return self
+
+    def _updateCS(self, adjust=True, scale=True):
+        "Update the current coordinate system for the resized canvas"
+        if self.coordSys is None:
+            raise AttributeError("{} has no coordinate system to update".format(self))
+        lrbt, m, (w0, h0) = self.coordSys._args
+        size = getattr(self, "scrollSize", self.size)
+        if scale: scale = size[0] / w0, size[1] / h0
+        if adjust is True: adjust = self.attr_get()
+        self.attachCS(lrbt, m)
+        if adjust: self.attr_set(adjust, scale)
+        return self
+
+    def resetCS(self):
+        "Remove the canvas coordinate system and reset to pixel coordinates"
+        u = lambda x: x
+        self._cs, self._px = u, u
+        self._units = 1, 1
+        self.coordSys = None
+        return self
+
     @property
     def border(self): return self._border
 
@@ -667,7 +855,8 @@ class Canvas(Graphic):
         "Calculate the clipping rect so as not to draw outside of the canvas"
         cv = self.canvas
         r = self.rect
-        return r.clip(cv.clipRect) if cv else r # cv.rect
+        if self.clipArea: r = r.clip(self.clipArea.move(*r.topleft))
+        return r.clip(cv.clipRect) if cv else r
 
     @property
     def angle(self): return 0
@@ -677,7 +866,7 @@ class Canvas(Graphic):
 
     @bg.setter
     def bg(self, bg): self._setBg(bg)
-    
+
     def _setBg(self, bg):
         t = type(bg)
         if t is str: bg = pygame.Color(bg)
@@ -718,11 +907,14 @@ class Canvas(Graphic):
         t = type(key)
         if not key.__hash__:
             raise KeyError("Type '{}' cannot be used as a key".format(t.__name__))
-        elif t in (int, slice):
+        elif t is slice:
             raise KeyError("Assignment by layer is not supported")
-#        gr._name = key
-#        if gr.canvas is not self:
-        if gr not in self: gr.setCanvas(self, key)
+        if gr not in self:
+            if t is int:
+                n = key
+                key = None
+            gr.setCanvas(self, key)
+            if t is int: gr.config(layer=n)
 
     def __iadd__(self, gr):
         "Add a Graphics instance(s) to the Canvas"
@@ -733,7 +925,8 @@ class Canvas(Graphic):
 
     def __isub__(self, gr):
         "Remove item(s) from the canvas"
-        if isinstance(gr, Graphic):
+        if type(gr) is str: self[gr].remove()
+        elif isinstance(gr, Graphic):
             if gr in self: gr.remove()
             else: raise ValueError("Cannot remove {} from {}".format(gr, self))
         else:
@@ -748,6 +941,14 @@ class Canvas(Graphic):
             gr = self[-1]
             if recursive and isinstance(gr, Canvas): gr.purge()
             gr.remove()
+        return self
+
+    def removeItems(self, *args):
+        "Remove specified items without raising exceptions if item is not in canvas"
+        for gr in args:
+            try: self -= gr
+            except: pass
+        return self
 
     def shiftContents(self, offset, *args, resize=True):
         "Move contents and (optionally) adjust canvas size"
@@ -757,21 +958,29 @@ class Canvas(Graphic):
                 x, y = gr.pos
                 gr.pos = x + dx, y + dy
         if resize:
-            self._size = self._size[0] + dx,self._size[1] + dy
+            self._size = self._size[0] + dx, self._size[1] + dy
         return self
 
-    def resize(self, size, resizeContent=True):
+    def resize(self, size, resizeContent=None):
         "Resize the canvas contents"
         size = max(1, round(size[0])), max(1, round(size[1]))
         fx, fy = size[0] / self._size[0], size[1] / self._size[1]
         self._size = size
 
+        # Resize scroll size
+        if getattr(self, "resizeScroll", False):
+            w, h = self.scrollSize
+            self.scrollSize = round(fx * w), round(fy * h)
+
         # Resize content
+        if resizeContent is None: resizeContent = self.resizeContent  
         if resizeContent:
-            for g in self:
-                if g.autoPositionOnResize: g.scaleVectors(fx, fy)
-                w, h = g.size
-                g.resize((w * fx, h * fy))
+            if self.coordSys: self._updateCS()
+            else:
+                for g in self:
+                    if g.autoPositionOnResize: g.scaleVectors(fx, fy)
+                    w, h = g.size
+                    g.resize((w * fx, h * fy))
 
     def draw(self, srf=None, mode=3):
         "Draw the canvas to a surface"
@@ -795,6 +1004,7 @@ class Canvas(Graphic):
         if mode & 2:
             br = isSketch and self.dirtyRegions is not None
             if br: self.dirtyRegions = []
+            ondrawList = self.sketch.ondrawList
             for g in list(self):
                 srf.set_clip(self.clipRect)
                 if not hasattr(g, "image") and g.effects:
@@ -805,8 +1015,7 @@ class Canvas(Graphic):
                 else: grect = g.draw(srf)
                 g.rect = grect
                 if br: self.dirtyRegions.append(grect)
-#                if g.ondraw and g.ondraw(): g.remove()
-                if g.ondraw: g.ondraw()
+                if g.ondraw: ondrawList.append(g) # g.ondraw()
 
         # Draw border
         if mode & 1 and self.weight:
@@ -830,19 +1039,28 @@ class Canvas(Graphic):
             if g.snapshot is not None:
                 img = g.snapshot().image
                 srf.blit(img, g.blitPosition((0,0), img.get_size()))
-#                xy = g.blitPosition((0,0), g.size)
-#                srf.blit(g.snapshot().image, xy)
             else: g.draw(srf, snapshot=True)
 
         # Draw border
         if self.weight: drawBorder(srf, self.border, self.weight)
         return Image(srf)
 
-    def objectAt(self, pos):
+    def flatten(self, keep=()):
+        "Draw graphics onto background and remove"
+        if isinstance(keep, Graphic): keep = (keep,)
+        keep = [(gr.name, gr) for gr in keep]
+        for gr in keep: gr[1].remove()
+        self.config(bg=self.snapshot()).purge()
+        for name, gr in keep:
+            if name: self[name] = gr
+            else: self += gr
+        return self
+
+    def objectAt(self, pos, includeAll=False):
         obj = self
         for g in self:
             try: # Objects added but not yet blitted have no rect
-                if g.contains(pos):
+                if (includeAll or g.hoverable) and g.contains(pos):
                     obj = g.objectAt(pos) if isinstance(g, Canvas) else g
             except: pass
         return obj
@@ -866,33 +1084,39 @@ class Canvas(Graphic):
         for gr in (self.everything() if recursive else self):
             if criteria(gr): yield gr
 
-    @staticmethod
-    def grid(*args, cols=None, size=None):
-        "Arrange graphics in a grid"
-        n = len(args)
-        if cols is None: cols = n
-        rows = n // cols
-        if rows * cols < n: rows += 1
-        args = [(a if isinstance(a, Graphic) else Image(a)) for a in args]
-        w, h = size if size else args[0].size 
-        cv = Canvas((w*cols, h*rows))
-        r = c = 0
-        for i in range(n):
-            cv += args[i].config(anchor=TOPLEFT, pos=(c*w, r*h), size=(w, h))
-            c += 1
-            if c == cols:
-                c = 0
-                r += 1
-        return cv
+    def scroll(self, dx=0, dy=0):
+        raise NotImplementedError("Use ScrollCanvas class.")
+
+    def cover(self):
+        return Image(self.size, "#ffffffc0").config(anchor=TOPLEFT)
+
+    def attr_get(self, *args):
+        data = {}
+        for gr in self:
+            d = {}
+            for a in (args if args else gr._preserve):
+                try: d[a] = getattr(gr, a)
+                except: pass
+            if d: data[gr] = d
+        return data
+
+    def attr_set(self, data, scale=None):
+        for gr, d in data.items():
+            if scale and "size" in d:
+                d = d.copy()
+                d["size"] = vmult(d["size"], scale)
+            gr._config(0, **d)
 
 
 class Sketch(Canvas):
+    minSize = 32
     capture = None
     realTime = False
     frameRate = 60
-    anchor = 0
     _fixedAspect = True
     dirtyRegions = []
+    resizeTrigger = False
+#     beforeResize = None
 
     def __init__(self, size=(512,288)):
         super().__init__(size, "white")
@@ -900,6 +1124,18 @@ class Sketch(Canvas):
         self.frameCount = 0
         self.evMgr = EventManager(self)
 
+    @property
+    def focusable(self): return True
+
+    @property
+    def hoverable(self): return True
+
+    @property
+    def pos(self): return 0, 0
+
+    @property
+    def anchor(self): 0
+
     @property
     def caption(self): return _pd.get_caption()[0]
 
@@ -907,9 +1143,6 @@ class Sketch(Canvas):
     def caption(self, caption):
         return _pd.set_caption(caption)
 
-    @property
-    def focusable(self): return True
-
     @property
     def timeFactor(self):
         if self.frameCount == 1: return 1
@@ -926,6 +1159,8 @@ class Sketch(Canvas):
         self._setBg(bg)
         if self._fixedAspect and hasattr(bg, "aspectRatio"):
             self._fixedAspect = bg.aspectRatio
+            size = self.size
+            self.resize(self._aspectSize(size, size))
         if self.dirtyRegions is not None:
             self.dirtyRegions = [pygame.Rect((0,0), self._size)]
 
@@ -942,9 +1177,6 @@ class Sketch(Canvas):
         if fn is None: fn = "save/img{:05d}.png".format(self.frameCount)
         pygame.image.save(self.image, fn)
 
-    def cover(self):
-        return Image(self.size, "#ffffffc0").config(anchor=TOPLEFT)
-
 # Resizing methods
 
     @property
@@ -961,8 +1193,6 @@ class Sketch(Canvas):
 
     @fixedAspect.setter
     def fixedAspect(self, a):
-#         self._fixedAspect = a
-#         if a: self.resize(self.size)
         if a:
             w, h = self.size
             self._fixedAspect = w / h
@@ -982,23 +1212,32 @@ class Sketch(Canvas):
 
     def _pygameMode(self, n): return pygame.RESIZABLE if n is True else int(n)
 
-    def resize(self, size, mode=None):
+    def resize(self, size, mode=None): # Rewritten (non-recursive!) for v2.2.a3 !!!
         "Resize the sketch, maintaining aspect ratio if required"
+        initSize = self._size
+        ms = self.minSize
+        if self._fixedAspect:
+            size = self._aspectSize(size, initSize)
+            f = max(1, ms / min(size))
+            size = round(f * size[0]), round(f * size[1])
+        else:
+            size = max(ms, round(size[0])), max(ms, round(size[1]))
         if mode is None: mode = self._mode
         else:
             mode = self._pygameMode(mode)
             self._mode = mode
-        initSize = self.size
-        size = round(size[0]), round(size[1])
         self.image = _pd.set_mode(size, mode)
-        _pd.flip()
-        if self._fixedAspect: size = self._aspectSize(size, initSize)
-        if self._fixedAspect and sum(abs(x-y) for (x,y) in (zip(size, self.size))) > 1:
-            return self.resize(size)
+#         if self.beforeResize: self.beforeResize(initSize)
         super().resize(self.size)
         self._size = self.size
         if self.dirtyRegions is not None:
             self.dirtyRegions = [pygame.Rect((0,0), self._size)]
+        evMgr = self.evMgr
+        ev = getattr(self, "_resize_ev", None)
+        self._resize_ev = None
+        if ev is None and self.resizeTrigger:
+            ev = pygame.event.Event(pygame.USEREVENT, focus=evMgr.focus, hover=evMgr.hover)
+        if ev: evMgr.handle(self, "onresize", ev)
 
 # Drawing methods
 
@@ -1011,9 +1250,9 @@ class Sketch(Canvas):
         pygame.key.set_repeat(400, 80)
         _pd.set_caption(caption)
         try:
-            try: icon = pygame.image.load(icon)
-            except: icon = Image.fromBytes(sc8prData("alien")).image 
-            _pd.set_icon(icon)
+            try: icon = Image(icon) # pygame.image.load
+            except: icon = Image.fromBytes(sc8prData("alien"))
+            _pd.set_icon(icon.config(width=64).image)
         except: logError()
         w, h = self._size
         if self._fixedAspect: self._fixedAspect = w / h
@@ -1026,6 +1265,9 @@ class Sketch(Canvas):
 
         # Run setup
         try:
+            if hasattr(self, "_defer_coords"):
+                self.setCoords(*self._defer_coords)
+                del self._defer_coords
             if hasattr(self, "setup"): self.setup()
             else:
                 main = sys.modules["__main__"]
@@ -1038,6 +1280,7 @@ class Sketch(Canvas):
                 self.frameCount += 1
                 br = self.dirtyRegions
                 flip = br is None
+                self.ondrawList = []
                 self.draw()
                 if not flip:
                     br += self.dirtyRegions
@@ -1045,7 +1288,10 @@ class Sketch(Canvas):
                 self._clock.tick(self.frameRate)
                 if flip: _pd.flip()
                 else: _pd.update(br)
-                if self.capture is not None: self.capture.capture(self)
+                self._capture()
+                for gr in self.ondrawList:
+                    try: gr.ondraw()
+                    except: logError()
                 if self.ondraw: self.ondraw()
                 self._evHandle()
             except: logError()
@@ -1055,6 +1301,14 @@ class Sketch(Canvas):
         if mod: mod.Font.dumpCache()
         return self
 
+    def _capture(self):
+        "Call screen capture method when recording"
+        c = self.capture
+        if c is not None:
+            i = getattr(c, "interval", 1)
+            if self.frameCount % i == 0:
+                c.capture(self)
+
     def _evHandle(self):
         "Handle events in the pygame event queue"
         resized = False
@@ -1070,10 +1324,7 @@ class Sketch(Canvas):
                         resized = True
                         setattr(ev, "originalSize", self._size)
                         self._resize_ev = ev
-                        s = hasattr(self, "_scrollSize")
-                        if s: self.scrollTo()
                         self.resize(size)
-                        if s: self.resizeCoords(ev)
             except: logError()
 
     def _drawDirtyRegions(self, srf):
