@@ -1,4 +1,4 @@
-# Copyright 2015-2019 D.G. MacCarthy <http://dmaccarthy.github.io>
+# Copyright 2015-2021 D.G. MacCarthy <http://dmaccarthy.github.io>
 #
 # This file is part of "sc8pr".
 #
@@ -15,11 +15,11 @@
 # You should have received a copy of the GNU General Public License
 # along with "sc8pr".  If not, see <http://www.gnu.org/licenses/>.
 
-
 import pygame
 
 class EventManager:
-    "A class for inspecting and dispatching events"
+    "Process events (except VIDEORESIZE) from the pygame event queue"
+#     debug = 0
 
     def __init__(self, sk):
         self.sk = sk
@@ -28,7 +28,10 @@ class EventManager:
         self.drag = None
 
     def dispatch(self, ev):
-        # Record mouse and keyboard events
+        "Process one pygame event"
+#         if self.debug: print("Procesing  :", pygame.event.event_name(ev.type) if self.debug == 1 else ev)
+
+        # Encapsulate mouse and keyboard events as sketch attributes
         sk = self.sk
         other = False
         key = hasattr(ev, "key")
@@ -36,97 +39,101 @@ class EventManager:
         elif hasattr(ev, "pos"): sk.mouse = ev
         else: other = True
 
-        # Trigger sk.onevent
-        if hasattr(sk, "onevent"):
-            if sk.onevent(ev): return
-
-        # Get event target and handler name
+        # Determine 'hover' graphic
         path = sk.objectAt(sk.mouse.pos).path
         if not path: path = [sk]
-        setattr(ev, "target", path[0])
+        self._oldHover = self.hover
+        self.hover = path[0]
+        setattr(ev, "hover", path[0])
+    
+        # Set 'focus' graphic and handler name
+        setattr(ev, "focus", self.focus)
         name = "on" + pygame.event.event_name(ev.type).lower()
 
-        # Send events other than mouse or keyboard to the sketch handler
-        if other:
-            name = name.replace("event", "")
-            self.handle(sk, name, ev)
+        # Call sk.onevent
+        if hasattr(sk, "onevent"):
+            setattr(ev, "handler", None)
+            if sk.onevent(ev): return
+
+        # Send non-mouse and non-keyboard events
+        # to the appropriate sketch event handler
+        if other: self.handle(sk, name.replace("event", ""), ev)
 
         # Send KEYDOWN and KEYUP events to the focused object
-        elif key:
-            current = self.focus
-            if current is None: current = sk
-            setattr(ev, "target", current)
-            self.handle(current.path, name, ev)
+        elif key: self.handle(self.focus, name, ev)
 
-        # Send MOUSEDOWN events to the onblur and onclick methods of the
-        # respective objects. Set focus if a focusable object is clicked
+        # Process MOUSEBUTTONDOWN events by calling as necessary:
+        # onrelease, onblur, onfocus, onmousedown
         elif ev.type == pygame.MOUSEBUTTONDOWN:
+            if self.drag is not None: self._dragRelease(ev)
             for p in path:
                 if p.focusable:
                     focus = p
                     break
-            if self.drag is not None: self._dragRelease(ev)
-            if self.focus is not focus and hasattr(self.focus, "onblur"):
-                setattr(ev, "target", self.focus)
-                getattr(self.focus, "onblur")(ev)
-                setattr(ev, "target", path[0])
-            self.focus = focus
-            self.handle(path, "onclick", ev)
+            if self.focus is not focus:
+                setattr(ev, "focus", focus)
+                self.handle(self.focus, "onblur", ev)
+                self.focus = focus
+                self.handle(focus, "onfocus", ev)
+            self._mousedown = path[0]
+            self.handle(path, "onmousedown", ev)
 
-        # Send MOUSEUP events to the object bring dragged
+        # Process MOUSEBUTTONUP events by calling onrelease or onmouseup/onclick
         elif ev.type == pygame.MOUSEBUTTONUP:
             if not self._dragRelease(ev):
                 self.handle(path, "onmouseup", ev)
+                if path[0] is self._mousedown:
+                    self.handle(path, "onclick", ev)
 
-        # Send MOUSEMOTION events to the onmouseout and onmouseover
-        # methods of the respective objects; trigger ondrag method
-        # if an object is begin dragged
+        # Process MOUSEMOTION events by calling as necessary:
+        # onmouseout, onmouseover, ondrag, onmousemotion
         elif ev.type == pygame.MOUSEMOTION:
             self._overOut(path, ev)
             drag = False
             if sum(ev.buttons): # Dragging
-                if self.drag is not None: hoverPath = self.drag.path
-                else: hoverPath = self.hover.path
-                current = _find(hoverPath, "ondrag")
+                current = _find(self._oldHover.path, "ondrag") \
+                    if self.drag is None else self.drag
                 if current is not None:
-                    if self.drag is not current:
-                        self.drag = current
-                    getattr(current, "ondrag")(ev)
+                    if self.drag is not current: self.drag = current
+                    self.handle(current, "ondrag", ev)
                     drag = True
             if not drag: self.handle(path, "onmousemotion", ev)
 
-        # Trigger sk.onhandled
-        self.hover = path[0]
-        if hasattr(sk, "onhandled"): sk.onhandled(ev)
+        # Call sk.onhandled
+        if hasattr(sk, "onhandled"):
+#             delattr(ev, "target")  # Removed!!!
+            sk.onhandled(ev)
 
     def handle(self, path, eventName, ev):
         "Locate and call the appropriate event handler"
         if type(path) is not list: path = path.path
+        setattr(ev, "target", path[0])
+        setattr(ev, "handler", eventName)
         current = _find(path, eventName)
-        if current is None: current = path[0]
-        if hasattr(current, eventName):
-            return getattr(current, eventName)(ev)
+        handle = current is not None
+#         if self.debug:
+#             print("Handling  :" if handle else "No handler:", pygame.event.event_name(ev.type) if self.debug == 1 else ev)
+        if handle: getattr(current, eventName)(ev)
 
     def _dragRelease(self, ev):
+        "Handle RELEASE events for graphic being dragged"
         drag = self.drag
-        if drag:
-            if hasattr(drag, "onrelease"):
-                getattr(drag, "onrelease")(ev)
+        if drag is not None:
+            self.handle(drag, "onrelease", ev)
             drag = True
         else: drag = False
         self.drag = None
         return drag
 
     def _overOut(self, path, ev):
-        oldHover = self.hover.path
+        "Handle MOUSEOVER and MOUSEOUT events"
+        oldHover = self._oldHover.path
         obj = oldHover[0]
         while obj not in path:
-            setattr(ev, "target", obj)
             self.handle(obj, "onmouseout", ev)
             obj = obj.canvas
         i = path.index(obj)
         for obj in reversed(path[:i]):
-            setattr(ev, "target", obj)
             self.handle(obj, "onmouseover", ev)
 
 
