@@ -1,4 +1,4 @@
-# Copyright 2015-2018 D.G. MacCarthy <http://dmaccarthy.github.io>
+# Copyright 2015-2021 D.G. MacCarthy <http://dmaccarthy.github.io>
 #
 # This file is part of "sc8pr".
 #
@@ -16,9 +16,12 @@
 # along with "sc8pr".  If not, see <http://www.gnu.org/licenses/>.
 
 
+import pygame
 from pygame.constants import K_LEFT, K_RIGHT, K_UP, K_DOWN
 from sc8pr import Canvas, Image, Graphic
+from sc8pr.sprite import CostumeImage
 from sc8pr.util import tall
+from sc8pr.geom import sigma, delta
 
 CLICK = 0
 SCROLL = 1
@@ -28,28 +31,49 @@ KEY = 3
 def _knobDrag(knob, ev):
     "Handle drag events on the slider's 'knob' object"
     slider = knob.canvas
+    if knob._dragRel is None:
+        knob._dragRel = delta(knob.rect.center, ev.pos)
     if slider._lastButton in slider.allowButton:
-        x = slider._eventValue(ev)
+        x = pygame.event.Event(pygame.USEREVENT, pos=sigma(ev.pos, knob._dragRel))
+        v = slider._val
+        slider.val = slider._eventValue(x)
+        x = slider.val
         setattr(ev, "target", slider)
-        if x != slider._val:
-            slider.val = x
+        if x != v:
             setattr(ev, "method", DRAG)
             slider.bubble("onchange", ev)
 
+def _knobRelease(knob, ev): knob._dragRel = None
+
+
+class Knob(CostumeImage):
+    colors = "#bdbdbd", "#a0a0a0"
+    
+    def __init__(self, size, colors=None):
+        if colors is None: colors = self.colors
+        try: imgs = [Image(size, c) for c in colors]
+        except: imgs = [Image(size, colors)]
+        super().__init__(imgs)
+
+    def ondraw(self):
+        hov = self is self.sketch.evMgr.hover
+        self.costumeNumber = 1 if hov else 0 # or self.canvas.focussed
+
 
 class Slider(Canvas):
     "A numerical slider GUI control"
+    _lastButton = None
     focusable = True
     allowButton = 1, 4, 5
-
-    def __init__(self, size=(128,16), knob="grey", lower=0, upper=1, steps=0):
+    reverseWheel = False
+    _methodNames = ["Click", "Scroll", "Drag", "Key"]
+    
+    def __init__(self, size=(128,16), knob=None, lower=0, upper=1, steps=0):
         super().__init__(size)
-        if not isinstance(knob, Graphic): knob = Image(bg=knob)
-        w, h = size
-        kwargs = {"width":w} if h > w else {"height":h}
-        self += knob.bind(ondrag=_knobDrag).config(**kwargs)
-        self.knob = knob
         self.steps = steps
+        if not isinstance(knob, Graphic): knob = Knob(self._knobSize(), knob)
+        self += knob.bind(ondrag=_knobDrag, onrelease=_knobRelease)
+        self.knob = knob.config(_dragRel=None)
         if upper < lower:
             self._flip = True
             upper, lower = lower, upper
@@ -58,6 +82,19 @@ class Slider(Canvas):
         self.upper = upper
         self.val = lower
 
+    @property
+    def methodNames(self): return self._methodNames
+
+    def _knobSize(self):
+        w, h = self.size
+        a = min(w, h)
+        if self.steps:
+            b = round(max(w, h) / (1 + self.steps)) 
+            b = max(a, b)
+            w, h = (a, b) if h > w else (b, a)
+        else: w = h = a
+        return w, h
+
     @property
     def val(self): return self._val
 
@@ -70,7 +107,7 @@ class Slider(Canvas):
         if self._flip: x = 1 - x
         knob = self.knob
         dim = tall(*self.size)
-        wh = knob.size[dim] + 2
+        wh = knob.size[dim] #+ 2
         w, h = self.size
         knob.pos = (w / 2, wh / 2 + (h - wh) * x) if dim else (wh / 2 + (w - wh) * x, h / 2)
 
@@ -84,17 +121,18 @@ class Slider(Canvas):
     def _eventValue(self, ev):
         "Determine a numerical value from the event coordinates"
         dim = tall(*self.size)
-        wh = self.knob.size[dim] + 2
-        x = self._round((self.relXY(ev.pos)[dim] - wh / 2) / (self.size[dim] - wh))
+        wh = self.knob.size[dim] #+ 2
+        x = self._round((self.relXY(ev.pos)[dim] - wh / 2) / max(1, self.size[dim] - wh))
         if self._flip: x = 1 - x
         return self._calc(x)
 
-    def onclick(self, ev):
-        "Handle click events on the slider canvas"
+    def onmousedown(self, ev):
+        "Handle MOUSEDOWN events on the slider canvas"
         self._lastButton = btn = ev.button
         if btn in self.allowButton:
             dim = tall(*self.size)
             lims = [self.lower, self.upper]
+            if btn in (4, 5) and self.reverseWheel: btn = 9 - btn
             if btn == 4: x = lims[dim]
             elif btn == 5: x = lims[1-dim]
             else: x = self._eventValue(ev)
@@ -103,7 +141,8 @@ class Slider(Canvas):
             s = (self.upper - self.lower) / s
             v = self._val
             if x != v:
-                if x < v: self.val -= s
+                if btn == 1 and ev.target is self: self.val = x
+                elif x < v: self.val -= s
                 else: self.val += s
                 setattr(ev, "method", SCROLL if btn in (4,5) else CLICK)
                 if ev: self.bubble("onchange", ev)
@@ -123,3 +162,6 @@ class Slider(Canvas):
             if self._val != cur:
                 setattr(ev, "method", KEY)
                 self.bubble("onchange", ev)
+
+    def resize(self, size):
+        super().resize((16, size[1]))
