@@ -1,4 +1,4 @@
-# Copyright 2015-2018 D.G. MacCarthy <https://dmaccarthy.github.io/sc8pr>
+# Copyright 2015-2021 D.G. MacCarthy <https://dmaccarthy.github.io/sc8pr>
 #
 # This file is part of "sc8pr".
 #
@@ -18,6 +18,7 @@
 
 from random import uniform, random
 from math import sqrt, tan, pi, sin, cos, hypot
+from sys import stderr
 import pygame
 from pygame.pixelarray import PixelArray
 from sc8pr.util import rgba, style
@@ -25,15 +26,27 @@ from sc8pr.util import rgba, style
 pi2 = 2 * pi
 
 
+def shift(gr, dt):
+    "Shift effect timing"
+    for e in gr.effects:
+        try:
+            t0, t1 = e._time
+            e._time = t0 + dt, t1 + dt
+        except: pass
+    return gr
+
+
 class Effect:
     "Base class for all effects / transitions"
     _time = None
 
     def time(self, fullEffect, noEffect=0):
+        "Set effect timing"
         self._time = fullEffect, noEffect
         return self
 
     def transition(self, img, n):
+        "Apply an effect as a transition in/out"
         if self._time is None: n = 0
         else:
             t0, t1 = self._time
@@ -48,13 +61,14 @@ class Effect:
 class Remove(Effect):
     "Remove a graphic, or its effects lists, at the specified frame"
 
-    def __init__(self, gr, complete=True):
+    def __init__(self, gr, complete=False):
         self._gr = gr
         self._complete = complete
 
     def apply(self, img, n=0):
         if n <= 0:
-            if self._complete: self._gr.remove()
+            if self._complete:
+                self._gr.bind(ondraw=lambda gr:gr.remove())
             else: self._gr.effects = None
         return img
 
@@ -182,11 +196,10 @@ class Squash(Wipe):
 class MathEffect(Effect):
     "Effect based on y < f(x) or y > f(x)"
 
-    def __init__(self, noise=0.15, fill=(0,0,0,0)):#, eqn=None):
+    def __init__(self, noise=0.15, fill=(0,0,0,0)):
         self.fill = rgba(fill)
         self.above = noise > 0
         self.noise = abs(noise)
-#        if eqn: self.eqn = eqn.__get__(self, self.__class__)
 
     def eqn(self, x, n, size):
         dh = self.noise
@@ -332,3 +345,50 @@ class ClockHand(MathEffect):
             return 0 if n <= 0.5 else (y + x * tan((n - 0.75) * pi2), False)
         else:
             return h if n >= 0.5 else (y + x * tan((n - 0.25) * pi2))
+
+
+class Pixelate(Effect):
+
+    def __init__(self, size=64, linear=True):
+        if linear: # Better for smaller squares?
+            self._calc = lambda x: max(1, round((1 - x) * size))
+        else:      # Better for larger squares?
+            self._calc = lambda x: round(size ** (1 - x))
+
+    def apply(self, img, n=0):
+        if n >= 1: return img
+        srf, (w, h) = self.srfSize(img, True)
+        n = self._calc(n)
+        if n > 1:
+            r = pygame.Rect(0, 0, w, h)
+            dx = n - (w % n) // 2
+            dy = n - (h % n) // 2
+            for x in range(0, w + n, n):
+                for y in range(0, h + n, n):
+                    clip = r.clip(x - dx, y - dy, n, n)
+                    if clip.width and clip.height:
+                        subsrf = srf.subsurface(clip)
+                        c = pygame.transform.average_color(subsrf)
+                        subsrf.fill(c)
+        return srf
+
+
+try:
+    from pygame.surfarray import pixels_alpha
+    
+    class FastDissolve(Effect):
+        """Dissolve to transparency only; uses pygame.surfarray for speed but requires numpy"""
+    
+        def apply(self, img, n):
+            if n >= 1: return img
+            srf = self.srfSize(img)
+            sa = pixels_alpha(srf)
+            h = len(sa[0])
+            for c in sa:
+                for r in range(h):
+                    if random() > n: c[r] = 0
+            return srf
+
+except:
+    FastDissolve = Dissolve
+    print("FastDissolve effect is unavailable; substituting Dissolve", file=stderr)
