@@ -1,4 +1,4 @@
-# Copyright 2015-2018 D.G. MacCarthy <http://dmaccarthy.github.io>
+# Copyright 2015-2021 D.G. MacCarthy <http://dmaccarthy.github.io>
 #
 # This file is part of "sc8pr".
 #
@@ -29,6 +29,16 @@ def hsva(h, s, v=100, a=100):
     c.hsva = h, min(100, s), v, a
     return c
 
+def hsvBox(color, hue=None, sat=None, val=None):
+    "Check if color is within specified color wheel 'box'"
+    h, s, v = pygame.Color(color).hsva[:3]
+    return (hue is None or _between(h, *hue)) and (sat is None
+        or _between(s, *sat)) and (val is None or _between(v, *val))
+
+def _between(x, x0, x1):
+    if x0 < x1: return x0 <= x and x <= x1
+    else: return x >= x0 or x <= x1
+
 def hs_surface(size=(100,100), v=100, wheel=False):
     "Create a hue-saturation color array"
     width, height = size
